<?php

namespace SGalinski\DfTabs\DataProvider;

/***************************************************************
 *  Copyright notice
 *
 *  (c) domainfactory GmbH (Stefan Galinski <stefan.galinski@gmail.com>)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\DfTabs\Exception\GenericException;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;

/**
 * Data Provider Factory
 */
final class FactoryDataProvider {
	/**
	 * Returns a table instance based upon the given type. The additional parameters are used
	 * to instantiate the table instance with needed information's.
	 *
	 * @throws GenericException if no valid data provider could be created
	 * @param string $type pages, tt_content, typoscript, ...
	 * @param array $pluginConfiguration
	 * @param ContentObjectRenderer $contentObject
	 * @return AbstractBaseDataProvider
	 */
	public static function getDataProvider($type, array $pluginConfiguration, ContentObjectRenderer $contentObject) {
		$dataProvider = NULL;
		if ($type === 'tt_content') {
			$dataProvider = GeneralUtility::makeInstance(ContentDataProvider::class);
		} elseif ($type === 'pages') {
			$dataProvider = GeneralUtility::makeInstance(PagesDataProvider::class);
		} elseif ($type === 'typoscript') {
			$dataProvider = GeneralUtility::makeInstance(TypoScriptDataProvider::class);
		} else {
			throw new GenericException('No data provider matched your request!');
		}

		/** @var $dataProvider AbstractBaseDataProvider */
		$dataProvider->injectPluginConfiguration($pluginConfiguration);
		$dataProvider->injectContentObject($contentObject);

		return $dataProvider;
	}
}

?>
